
; A simple public domain tool which turns requests for putting
; files into a PRINT queue into DOS calls which copy the to be
; printed file to a temp directory :-p. By Eric Auer 2007.
; Compile w/ "netwide ASM": "nasm -o prnfile.com prnfile.asm"

	cpu 8086
	org 100h

start:	mov ax,100h	; install check
	int 2fh
	cmp al,255	; already there
	jz noload
	cmp al,0	; not there yet (1 is "not ok to load")
	jnz nope2
	push es
	mov ax,352fh	; get int 2f
	int 21h
	mov [old2f],bx
	mov [old2f+2],es
	pop es
	mov ax,252fh	; hook int 2f
	mov dx,i2f	; offset of handler
	int 21h
	mov dx,hello
	mov ah,9	; show text
	int 21h
	push es
	mov es,[cs:2ch]	; environment
	mov ah,49h	; free memory
	int 21h
	pop es
	mov cl,4
	mov dx,hello+15	; size
	shr dx,cl
	mov ax,3100h	; exit and stay resident
	int 21h


noload:	mov dx,nopemsg
nold2:	mov ah,9	; show text
	int 21h
	mov ax,4c01h	; exit
	int 21h
nope2:	mov dx,nope2m
	jmp short nold2

nopemsg	db "PRINT or prnfile already loaded",13,10,"$"
nope2m	db "Cannot load prnfile, int in use",13,10,"$"

eof:	; --- until here, things can be overwritten by buffer

i2f:	cmp ah,1	; our int 2f handler
	jz isprn
	db 0eah		; "jmp far"
old2f	dw 0,0
isprn:	cmp al,0	; install check
	jnz no0
	mov al,255	; already installed
	iret
no0:	cmp al,1	; enqueue file
	jz prn
	cmp al,2	; dequeue file named ds:dx
	jnz no2
bad9:	mov ax,9	; "spooler busy"
bad:	push bp
	mov bp,sp
	or byte [bp+6],1	; "stc"
	pop bp
	iret
no2:	cmp al,3	; dequeue all
	jz bad9
	cmp al,4	; view queue (stops printing until done)
	jnz no4
	mov ax,cs
	mov ds,ax	; segment of fake queue
	mov si,80h	; offset of fake queue: array of names
	mov byte [80h],0	; 64 byte each, array end is
	mov byte [80h+64],0	; marked by an empty name...
	xor dx,dx	; error count since last call: no error
	jmp short good
no4:	cmp al,5	; "done with view queue"
	jz good		; a real PRINT would go on printing now
	cmp al,6	; get header of used printer device
	; none :-)	; not "busy", so return no ds:si :-p
	jz good		; if any files now printing: stc, ax=8
	mov ax,1
	jmp short bad	; invalid function

badprn:	pop ds
	pop dx
	pop cx
	pop bx
	pop bp
	jmp short bad

	; we just assume that we are not called from within any
	; timing critical place or interrupt or similar...
prn:	; ds:dx -> byte 0/1 "form feed flag", then filename ptr
	; ... if ok: clc, return al=1 enqueued / 9e now printing
	; ... if not ok: stc, ax=1..f various error codes
	; (1 inv func, 2 file / 3 path / f drive not found...)
	push bp
	push bx
	push cx
	push dx
	push ds
	inc dx		; ignore form feed flag
	mov bx,dx
	lds dx,[ds:bx]	; fetch pointer to file name
	mov ax,3d00h	; open file for reading
	int 21h		; returns handle AX
	jc badprn	; error codes match PRINT ones ;-)
	mov bx,ax	; source file handle
	mov ax,cs
	mov ds,ax
mkprn:	mov ah,5bh	; create file (do not overwrite)
	xor cx,cx	; no attributes set
	mov dx,myname	; target file name
	int 21h
	jnc canprn
	cmp ax,50h	; is error "file exists"?
	jnz badprn
	call mynext	; try next name
	jmp short mkprn	; infinite loop if all 1000 exist :-p

canprn:	mov bp,ax	; target file handle
	call mynext	; increment target file name
	mov dx,80h	; our buffer
write:	mov cx,eof-80h	; buffer size
	and cx,0fff0h	; round down to multiple of 16
	or ch,ch	; use at most 256 bytes :-p
	jz abit		; if less, use as much as available
	mov cx,256	; stay humble ;-)
abit:	mov ah,3fh	; read file
	int 21h
	jc badprn
	or ax,ax	; at eof?
	jnz copyme
done:	mov ah,3eh	; close source file
	int 21h
	jc badprn
	mov bx,bp	; close target file
	mov ah,3eh
	int 21h
	jc badprn
	pop ds
	pop dx
	pop cx
	pop bx
	pop bp

good:	xor ax,ax
	push bp
	mov bp,sp
	and byte [bp+6],0feh	; "clc"
	pop bp
	iret

copyme:	mov cx,ax	; byte count
	xchg bx,bp	; handle: target
	mov ah,40h	; write file
	int 21h
	xchg bx,bp	; handle: source
	jc badprn
	cmp cx,ax	; all bytes written?
	jnz badprn
	jmp short write	; continue with more bytes to write

mynext:	push ax		; calculate next file name
	mov ax,[myname+6]	; last 2 digits
	inc ah		; count 1 digit
	cmp ah,'9'+1
	jnz mydone
	mov ah,'0'	; 0 again
	inc al		; count 10 digit
	cmp al,'9'+1
	jnz mydone
	inc byte [myname+5]	; count 100 digit
	; no check	; bad luck if you overflow here :-p
mydone:	mov [myname+6],ax	; do not forget to update
	pop ax
	ret

myname	db "print000.txt",0

hello	db "Prnfile loaded - Public Domain by Eric Auer 2007"
	db 13,10
	db "Fake PRINT: redirects to print*.txt in caller dir"
	db 13,10,"$"
